import { IObject } from '../adapter';
import { Agent } from '../agent';
import { CallerBroker } from '../brokers';
declare const ILSymbol: unique symbol;
export declare class BatchedCallerAgent<T extends IObject> extends Agent<CallerBroker> implements ProxyHandler<T>, PromiseLike<any> {
    protected readonly il: number;
    protected readonly instructions: BatchedCallerAgent.Instruction[];
    constructor(options: BatchedCallerAgent.Options);
    get [ILSymbol](): BatchedCallerAgent.Instruction.ILIntermediate;
    get(target: T, p: string | symbol): any;
    set(target: T, p: string | symbol, newValue: any): boolean;
    apply(target: T, thisArg: any, argArray: any[]): any;
    deleteProperty(target: T, p: string | symbol): boolean;
    construct(target: T, argArray: any[]): object;
    getPrototypeOf(): object;
    then<TResult1 = any, TResult2 = never>(onfulfilled?: (value: any) => TResult1 | PromiseLike<TResult1>, onrejected?: (reason: any) => TResult2 | PromiseLike<TResult2>): PromiseLike<TResult1 | TResult2>;
    toJSON(): BatchedCallerAgent.Instruction.ILIntermediate;
}
export declare namespace BatchedCallerAgent {
    interface Options {
        key: string;
        broker: CallerBroker;
        instructions?: BatchedCallerAgent.Instruction[];
        il?: number;
    }
    type Instruction = Instruction.Get | Instruction.Set | Instruction.Apply | Instruction.Del | Instruction.Ctor | Instruction.Return;
    namespace Instruction {
        type ILPrimitive = string | number | boolean | null;
        interface ILIntermediate {
            __il: number;
        }
        type ILValue = ILPrimitive | ILIntermediate | ILObject | ILArray;
        type ILObject = {
            [x: string | number]: ILValue;
        };
        type ILArray = Array<ILValue>;
        interface Get {
            t: 'get';
            il: number;
            p: string | number;
        }
        interface Set {
            t: 'set';
            il: number;
            p: string | number;
            v: ILValue;
        }
        interface Apply {
            t: 'apply';
            il: number;
            a?: ILValue[];
        }
        interface Del {
            t: 'del';
            il: number;
            p: string | number;
        }
        interface Ctor {
            t: 'ctor';
            il: number;
            a?: ILValue[];
        }
        interface Return {
            t: 'return';
            v?: ILValue;
        }
        function normalizeValue(raw: unknown): ILValue;
        function isILIntermediate(value: unknown): value is ILIntermediate;
        function reviveIL(ils: any[], value: unknown): any;
    }
}
export {};
