import { IAdapter, IObject } from '../adapter';
import { DeepAgent } from '../agent';
import { Broker } from '../broker';
import { BrokerMessage, BrokerResponse } from '../message';
import { IRegistrar } from '../registrar';
/**
 * Calls `CalleeBroker`
 */
export declare class CallerBroker extends Broker {
    protected readonly registrar: IRegistrar;
    protected readonly eventSubscribers: Map<string, Iterable<CallerBroker.EventHandler>>;
    protected readonly responseHandlers: Map<string, CallerBroker.ResponseHandler>;
    constructor(options: CallerBroker.Options);
    protected handleEvent(msg: BrokerMessage): void;
    protected handleResponse(msg: BrokerResponse): void;
    protected onAdapterMessage(msg: unknown): void;
    /**
     * Send a message without the need of a response
     * @param msg Message to send
     */
    send(msg: BrokerMessage): void;
    /**
     * Send a message with the need of a response
     * @param msg Message to send
     * @param cb Response handler
     */
    request(msg: BrokerMessage, cb: CallerBroker.ResponseHandler): void;
    /**
     * Send a message with the need of a response
     * @param msg Message to send
     * @returns Response
     */
    request<T = any>(msg: BrokerMessage): Promise<T>;
    /**
     * Subscribe to an event
     * @param type Event type
     * @param handler Event handler
     * @returns Unsubscribe function
     */
    subscribe(type: string, handler: CallerBroker.EventHandler): () => void;
    /**
     * Get an deferred agent
     * @param key Agent key
     */
    useAgent<T extends IObject>(key: string): DeepAgent<T>;
    /**
     * Run operations with batched agent
     * @param key Agent key
     * @param func Batch function
     * @returns Function return value
     */
    useAgent<T extends IObject, R>(key: string, func: (agent: T) => R): Promise<R>;
    /**
     * Reset data
     */
    reset(): void;
    dispose(): void;
}
export declare namespace CallerBroker {
    interface Options {
        /**
         * Adapter
         */
        adapter: IAdapter;
        /**
         * Registrar
         */
        registrar?: IRegistrar;
    }
    type EventHandler = (payload?: any) => void;
    type ResponseHandler = (error?: Error, payload?: any) => void;
}
